--Viewport.e
--Hawke' <mdeland@nwinfo.net> <trumpyt@nwinfo.net>
--9/21/98
--See viewport.txt for further description, usage
--rights, and version history.
--See viewport.ex for examples.
--Version 1.00B

global constant TRUE=1, FALSE=0
global constant PI = arctan(1) * 4

--800x600x256
--adjust the next three lines to your liking...
global constant SCR_MODE   = 259
global constant SCR_WIDTH  = 800
global constant SCR_HEIGHT = 600

global constant SCR_MAXX   = SCR_WIDTH  -1
global constant SCR_MAXY   = SCR_HEIGHT -1

global sequence WorldData, ViewportData
global atom     Xstep, Ystep --used in for loops as
                             --   "by Xstep" or "by Ystep"

sequence CoorListX, CoorListY
integer  PositiveGoesDown, --these are the way the screen
         PositiveGoesRight --normally is
atom     Xrange, Yrange
atom     Xoffset,Yoffset

global function FixBoundry(sequence data)
--This makes sure that the coordinate data {x,y}
--lies within the current screen resolution...
--It will adjust data if it isn't.
--It would be a good thing if you passed all your
--coordinate pairs thru this function first...
   if data[1] < 0        then data[1] = 0        end if
   if data[2] < 0        then data[2] = 0        end if
   if data[1] > SCR_MAXX then data[1] = SCR_MAXX end if
   if data[2] > SCR_MAXY then data[2] = SCR_MAXY end if
   return data
end function

global procedure SetWorld(sequence coor)
--Defines the new coordinate system via coor, which is
--{ {startx,starty},{endx,endy} }.

   WorldData = coor
   Xrange = WorldData[2][1] - WorldData[1][1]
   Yrange = WorldData[2][2] - WorldData[1][2]

   if Xrange >= 0 then
        PositiveGoesRight = TRUE
   else PositiveGoesRight = FALSE
   end if

   if Yrange >= 0 then
        PositiveGoesDown = TRUE
   else PositiveGoesDown = FALSE
   end if
end procedure

global procedure SetViewport(sequence coor)
--Defines the region of the actual screen to be plotted upon,
--via coor, which is { {startx,starty},{endx,endy} }.
--Accepts {} as { {0,0},{MAXY,MAXY} } which ensures a rectangle
--as large as the screen will allow.

sequence start,finish
   if length(coor) = 0 then 
        start  = {0,0}
        finish = {SCR_MAXY,SCR_MAXY}
   else start  = FixBoundry(coor[1])
        finish = FixBoundry(coor[2])
   end if
   ViewportData = {start,finish}
   Xoffset = start[1]
   Yoffset = start[2]
   Xstep = Xrange / (finish[1]-start[1])
   Ystep = Yrange / (finish[2]-start[2])

   --this is the heart of this whole affair...
   --a form of interpolation if you will, storing
   --all the possible values that reside at all the
   --pixel locations within the viewport, for fast
   --index lookups within CalcPosition()
   CoorListX = {}   CoorListY = {}
   for xval = WorldData[1][1] to WorldData[2][1] by Xstep do
	CoorListX = CoorListX & xval
   end for
   for yval = WorldData[1][2] to WorldData[2][2] by Ystep do
	CoorListY = CoorListY & yval
   end for
end procedure

function SearchNormal(atom value,sequence data)
--Given a value to be transformed, and the transformation
--index/sequence, return the position within the index by
--comparing the value against the exact values within the
--index (via binary searching) and finding where that value
--would lie.
--In this case, Normal refers to a coordinate
--system where either right or down is increasing.

integer start,finish,half
   start  = 1
   finish = length(data)

   while (finish-start) >= 2 do
        half = floor( (finish-start) /2 ) + start
        if  (value >= data[half]) then
             start  = half
        else finish = half
        end if
   end while

   --ok... only 2 possibilities left...
   if (value >= data[start])  and
      (value <  data[finish]) then
        return start
   else return finish
   end if
end function

function SearchBkWard(atom value,sequence data)
--{see above}
--In this case, BkWard refers to a coordinate
--system where either left or up is increasing.

integer start,finish,half
   start  = 1
   finish = length(data)

   while (finish-start) >= 2 do
        half = floor( (finish-start) /2 ) + start
        if  (value <= data[half]) then
             start  = half
        else finish = half
        end if
   end while

   --ok... only 2 possibilities left...
   if (value <= data[start])  and
      (value >  data[finish]) then
        return start 
   else return finish
   end if
end function

global function CalcPosition(sequence coor)
--Takes a coordinate pair, spins it in a blender for a mild
--puree, and spits back an "appropriate" x,y point based
--upon the new coordinate system definitions.

integer plotX, plotY --(where on the screen it will appear)

   if PositiveGoesRight then
        plotX = SearchNormal(coor[1],CoorListX) + Xoffset
   else plotX = SearchBkWard(coor[1],CoorListX) + Xoffset
   end if

   if PositiveGoesDown then
        plotY = SearchNormal(coor[2],CoorListY) + Yoffset
   else plotY = SearchBkWard(coor[2],CoorListY) + Yoffset
   end if

   return {plotX,plotY}
end function

