--Viewport.e
--Hawke' <mdeland@nwinfo.net> <trumpyt@nwinfo.net>
--9/21/98
--See viewport.txt for further description, usage
--rights, and version history.
--Version 1.00B

include viewport.e
include graphics.e

object junk
atom now,before,after
integer frbef,fraft
sequence point

procedure DrawSection(sequence start, sequence finish,
                      sequence steps, integer color)
--Simply draws a square upon the screen; however,
--take note of the use of steps here, building
--a list of adjacent points to send to pixel()
sequence line,point
   line={}
   for x=start[1] to finish[1] by steps[1] do
      line=line & color
   end for
   for y=start[2] to finish[2] by steps[2] do
      --this is how we actually use the library to convert
      --our relative coordinate system into actual screen
      --pixel locations
      point = CalcPosition({start[1],y})
      pixel(line,point)
   end for
end procedure

--MAIN
junk = graphics_mode(SCR_MODE)

--this example shows a -PI to PI coordinate system and how
--we can then access those coordinates with values like PI/8
clear_screen()
SetWorld({{-PI,-PI},{PI,PI}})
SetViewport({})
DrawSection({0,-PI},{PI,0},{Xstep,PI/8},3)
while get_key()=-1 do end while

--this example is the same as the above, except instead of
--using the entire screen, we now have a smaller viewport
--that has a 100 pixel buffer around it
clear_screen()
SetWorld({{-PI,-PI},{PI,PI}})
SetViewport({ {100,100},{SCR_MAXY,SCR_MAXY}-100 })
DrawSection({0,-PI},{PI,0},{Xstep,PI/8},3)
while get_key()=-1 do end while

--this example shows a 'true' origin coordinate system
--whereby the x axis goes from -10(left) to 10(right)
--and the y axis goes from 10(top) to -10(bottom)
--it then fills all the quadrants with different colors
clear_screen()
SetWorld({{-10,10},{10,-10}})
SetViewport({})
   --top half
   DrawSection({-10,10},{ 0,  0},{Xstep,Ystep},3)
   DrawSection({  0,10},{10,  0},{Xstep,Ystep},4)
   --bottom half
   DrawSection({-10, 0},{ 0,-10},{Xstep,Ystep},5)
   DrawSection({  0, 0},{10,-10},{Xstep,Ystep},6)
while get_key()=-1 do end while

--same as above with a smaller (100 pixel buffer) viewport
clear_screen()
SetWorld({{-10,10},{10,-10}})
SetViewport({ {100,100},{SCR_MAXY,SCR_MAXY}-100 })
--top half
DrawSection({-10,10},{ 0,  0},{Xstep,Ystep},3)
DrawSection({  0,10},{10,  0},{Xstep,Ystep},4)
--bottom half
DrawSection({-10, 0},{ 0,-10},{Xstep,Ystep},5)
DrawSection({  0, 0},{10,-10},{Xstep,Ystep},6)
while get_key()=-1 do end while

--some benchmarking begins now. frbef is frameratebefore.
--this simply sets dots all over the place, times how long
--it takes and counts how many dots were set.
clear_screen()
frbef=0
now = time()
while get_key()=-1 do
   pixel( rand(7),rand( {SCR_MAXY,SCR_MAXY} ) )
   frbef=frbef+1
end while
before = time()-now

--this example is dual natured.  it shows that you can have
--any coordinate system you like, this one being a huge
--'universe' that might be applicable to gaming.
--it also benchmarks itself to compare against the
--prior example since it uses a "full screen" viewport.
--p.s. fraft is framerateafter
clear_screen()
SetWorld({{-10000,10000},{10000,-10000}})
SetViewport({}) 
fraft=0
now = time()
while get_key()=-1 do
   point=CalcPosition( rand({20000,20000}) -10000 )
   pixel(rand(7),point)
   fraft=fraft+1
end while
after = time()-now

--this is the same example as above, except it doesn't
--benchmark and it has a smaller viewport. if you were
--to actually time it, it would (should!) be faster than
--the prior example, as it has fewer pixels/points to
--deal with.
clear_screen()
SetWorld({{-10000,10000},{10000,-10000}})
SetViewport({ {100,100},{SCR_MAXY,SCR_MAXY}-100 })
while get_key()=-1 do
   point=CalcPosition( rand({20000,20000}) -10000 )
   pixel(rand(7),point)
end while

junk= graphics_mode(-1)
printf(1,"%11.1f pixels/sec before\n",frbef/before)
printf(1,"%11.1f pixels/sec after\n", fraft/after)
